@@ -8,8 +8,8 @@ void CondProbSampleReplace(int r, int c, double *p, int *conf, int *perm,
     int nans, int *ans, int *warn);
 
 /* used to be in R core... */
-void c_rcont2(int *nrow, int *ncol, int *nrowt, int *ncolt, int *ntotal,
-    double *fact, int *jwork, int *matrix);
+void c_rcont2(int nrow, int ncol, int *nrowt, int *ncolt, int ntotal,
+    double *fact, int *jwork, int **matrix);
 
 /* from rbn.c */
 void c_rbn_master(SEXP fitted, SEXP result, SEXP n, SEXP fix, int debuglevel);
